/*!
 * \file
 * p2p benchmarking executable.
 * Performs adaptive p2p benchmark between a pair of processors 0-1.
 * Arguments are described in \ref getopt.
 * 
 * \b p2p.plot draws the graph of the execution time (sec) against message size (kb).
 * - input: p2p.out
 * - output: p2p.eps (0-1 with error bars)
 * 
 * Using the gnuplot script:
 * \code
 * $ gnuplot collective.plot
 * \endcode
 */

#include "config.h"
#include "mpib.h"
#include <malloc.h>
#include <string.h>
#include <getopt.h>

int main(int argc, char** argv) {
	MPI_Init(&argc, &argv);
	MPI_Comm comm = MPI_COMM_WORLD;
#ifndef MPIB_DEBUG
	MPIB_Comm(MPI_COMM_WORLD, &comm);
	if (comm == MPI_COMM_NULL) {
		MPI_Finalize();
		return 0;
	}
#endif

	int exit;
	MPIB_getopt_help_default(&exit);
	MPIB_msgset msgset;
	MPIB_getopt_msgset_default(&msgset);
	//if we want adaptive message sizes, we should do:
	//msgset.stride = 0;
	MPIB_precision precision;
	MPIB_getopt_precision_default(&precision);
	int rank;
	MPI_Comm_rank(comm, &rank);
	if (rank == 0) {
		int c;
		char options[256] = "i:";
		strcat(options, MPIB_getopt_help_options());
		strcat(options, MPIB_getopt_msgset_options());
		strcat(options, MPIB_getopt_precision_options());
		while ((c = getopt(argc, argv, options)) != -1) {
			MPIB_getopt_help_optarg(c, "p2p", &exit);
			MPIB_getopt_msgset_optarg(c, &msgset);
			MPIB_getopt_precision_optarg(c, &precision);
		}
	}
	MPIB_getopt_help_bcast(&exit, 0, comm);
	if (exit) {
		MPI_Finalize();
		return 0;
	}
	MPIB_getopt_msgset_bcast(&msgset, 0, comm);
	MPIB_getopt_precision_bcast(&precision, 0, comm);

	if (rank == 0)
		printf("#p2p adaptive benchmark 0-1\n#\n");
	MPIB_print_processors(comm);
	if (rank == 0) {
		MPIB_print_msgset(msgset);
		MPIB_print_precision(precision);
	}
	MPIB_p2p_container* container;
	container = MPIB_Send_Recv_container_alloc();
	int count = 0;
	MPIB_result* results = NULL;
	double time = MPI_Wtime();
	MPIB_measure_p2p_msgset(container, comm, 0, 1, msgset, precision, &count, &results);
	if (rank == 0) {
		printf("#total benchmarking time: %le\n#\n", MPI_Wtime() - time);
		MPIB_print_result_th();
		int i;
		for (i = 0; i < count; i++)
			MPIB_print_result_tr(results[i]);
	}
	free(results);
	MPIB_p2p_container_free(container);

#ifndef MPIB_DEBUG
	MPI_Comm_free(&comm);
#endif
	MPI_Finalize();
	return 0;
}
